import logging
import pathlib
import re
import subprocess
from typing import NamedTuple

from auditwheel.error import InvalidLibc

LOG = logging.getLogger(__name__)


class MuslVersion(NamedTuple):
    major: int
    minor: int
    patch: int


def find_musl_libc() -> pathlib.Path:
    try:
        ldd = subprocess.check_output(["ldd", "/bin/ls"], errors="strict")
    except (subprocess.CalledProcessError, FileNotFoundError):
        LOG.error("Failed to determine libc version", exc_info=True)
        raise InvalidLibc

    match = re.search(
        r"libc\.musl-(?P<platform>\w+)\.so.1 "  # TODO drop the platform
        r"=> (?P<path>[/\-\w.]+)",
        ldd,
    )

    if not match:
        raise InvalidLibc

    return pathlib.Path(match.group("path"))


def get_musl_version(ld_path: pathlib.Path) -> MuslVersion:
    try:
        ld = subprocess.run(
            [ld_path], check=False, errors="strict", stderr=subprocess.PIPE
        ).stderr
    except FileNotFoundError:
        LOG.error("Failed to determine musl version", exc_info=True)
        raise InvalidLibc

    match = re.search(
        r"Version " r"(?P<major>\d+)." r"(?P<minor>\d+)." r"(?P<patch>\d+)", ld
    )
    if not match:
        raise InvalidLibc

    return MuslVersion(
        int(match.group("major")), int(match.group("minor")), int(match.group("patch"))
    )



import subprocess
from unittest.mock import patch

import pytest

from auditwheel.error import InvalidLibc
from auditwheel.musllinux import find_musl_libc, get_musl_version


@patch("auditwheel.musllinux.subprocess.check_output")
def test_find_musllinux_no_ldd(check_output_mock):
    check_output_mock.side_effect = FileNotFoundError()
    with pytest.raises(InvalidLibc):
        find_musl_libc()


@patch("auditwheel.musllinux.subprocess.check_output")
def test_find_musllinux_ldd_error(check_output_mock):
    check_output_mock.side_effect = subprocess.CalledProcessError(1, "ldd")
    with pytest.raises(InvalidLibc):
        find_musl_libc()


@patch("auditwheel.musllinux.subprocess.check_output")
def test_find_musllinux_not_found(check_output_mock):
    check_output_mock.return_value = ""
    with pytest.raises(InvalidLibc):
        find_musl_libc()


def test_get_musl_version_invalid_path():
    with pytest.raises(InvalidLibc):
        get_musl_version("/tmp/no/executable/here")


@patch("auditwheel.musllinux.subprocess.run")
def test_get_musl_version_invalid_version(run_mock):
    run_mock.return_value = subprocess.CompletedProcess([], 1, None, "Version 1.1")
    with pytest.raises(InvalidLibc):
        get_musl_version("anything")


@patch("auditwheel.musllinux.subprocess.run")
def test_get_musl_version_valid_version(run_mock):
    run_mock.return_value = subprocess.CompletedProcess([], 1, None, "Version 5.6.7")
    version = get_musl_version("anything")
    assert version.major == 5
    assert version.minor == 6
    assert version.patch == 7



import platform
from pathlib import Path

import pytest

from auditwheel.wheel_abi import analyze_wheel_abi

HERE = Path(__file__).parent.resolve()


@pytest.mark.skipif(platform.machine() != "x86_64", reason="only supported on x86_64")
@pytest.mark.parametrize(
    "file, external_libs",
    [
        ("cffi-1.5.0-cp27-none-linux_x86_64.whl", {"libffi.so.5"}),
        ("python_snappy-0.5.2-pp260-pypy_41-linux_x86_64.whl", {"libsnappy.so.1"}),
    ],
)
def test_analyze_wheel_abi(file, external_libs):
    winfo = analyze_wheel_abi(str(HERE / file))
    assert set(winfo.external_refs["manylinux_2_5_x86_64"]["libs"]) == external_libs


@pytest.mark.skipif(platform.machine() != "x86_64", reason="only supported on x86_64")
def test_analyze_wheel_abi_pyfpe():
    winfo = analyze_wheel_abi(str(HERE / "fpewheel-0.0.0-cp35-cp35m-linux_x86_64.whl"))
    assert (
        winfo.sym_tag == "manylinux_2_5_x86_64"
    )  # for external symbols, it could get manylinux1
    assert (
        winfo.pyfpe_tag == "linux_x86_64"
    )  # but for having the pyfpe reference, it gets just linux



import os
import sys
from pathlib import Path
from typing import List

import nox

RUNNING_CI = "TRAVIS" in os.environ or "GITHUB_ACTIONS" in os.environ


@nox.session(python=["3.6"], reuse_venv=True)
def lint(session: nox.Session) -> None:
    """
    Run linters on the codebase.
    """
    session.install("pre-commit")
    session.run("pre-commit", "run", "--all-files")


@nox.session()
def coverage(session: nox.Session) -> None:
    """
    Run coverage using unit tests.
    """
    session.install("-r", "test-requirements.txt", "pytest-cov", ".")
    session.run(
        "python",
        "-m",
        "pytest",
        "tests/unit",
        "--cov=auditwheel",
        "--cov-report=term-missing",
    )


def _docker_images(session: nox.Session) -> List[str]:
    tmp_dir = Path(session.create_tmp())
    script = tmp_dir / "list_images.py"
    images_file = tmp_dir / "images.lst"
    script.write_text(
        fr"""
import sys
from pathlib import Path
sys.path.append("./tests/integration")
from test_manylinux import MANYLINUX_IMAGES
images = "\n".join(MANYLINUX_IMAGES.values())
Path(r"{images_file}").write_text(images)
"""
    )
    session.run("python", str(script), silent=True)
    return images_file.read_text().splitlines()


@nox.session(python=["3.6", "3.7", "3.8", "3.9"])
def tests(session: nox.Session) -> None:
    """
    Run tests.
    """
    posargs = session.posargs
    session.install("-r", "test-requirements.txt", "-e", ".")
    if RUNNING_CI:
        session.install("pytest-cov", "codecov")
        posargs.extend(["--cov", "auditwheel", "--cov-branch"])
        # pull manylinux images that will be used.
        # this helps passing tests which would otherwise timeout.
        for image in _docker_images(session):
            session.run("docker", "pull", image, external=True)

    session.run("pytest", "-s", *posargs)
    if RUNNING_CI:
        session.run("auditwheel", "lddtree", sys.executable)
        try:
            session.run("codecov")
        except nox.command.CommandFailed:
            pass  # Ignore failures from codecov tool
